@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -25,20 +23,16 @@ definitions it links to.  If not, report the bug!
 """
 # https://tools.ietf.org/html/rfc3696
 
-from __future__ import absolute_import, division, print_function
-
 import os.path
 import string
 
-import hypothesis._strategies as st
-import hypothesis.internal.conjecture.utils as cu
+from hypothesis._settings import note_deprecation
 from hypothesis.errors import InvalidArgument
-from hypothesis.searchstrategy.strategies import SearchStrategy
-
-if False:
-    from typing import Text  # noqa
-    from hypothesis.searchstrategy.strategies import SearchStrategy, Ex  # noqa
-
+from hypothesis.internal.conjecture import utils as cu
+from hypothesis.internal.reflection import deprecated_posargs
+from hypothesis.strategies._internal import core as st
+from hypothesis.strategies._internal.ipaddress import ip_addresses
+from hypothesis.strategies._internal.strategies import SearchStrategy
 
 URL_SAFE_CHARACTERS = frozenset(string.ascii_letters + string.digits + "$-_.+!*'(),")
 
@@ -93,7 +87,7 @@ class DomainNameStrategy(SearchStrategy):
             1, 63, max_element_length, "max_element_length"
         )
 
-        super(DomainNameStrategy, self).__init__()
+        super().__init__()
         self.max_length = max_length
         self.max_element_length = max_element_length
 
@@ -121,7 +115,7 @@ class DomainNameStrategy(SearchStrategy):
             .flatmap(
                 lambda tld: st.tuples(
                     *[st.sampled_from([c.lower(), c.upper()]) for c in tld]
-                ).map(u"".join)
+                ).map("".join)
             )
         )
         # The maximum possible number of subdomains is 126,
@@ -141,11 +135,10 @@ class DomainNameStrategy(SearchStrategy):
 
 
 @st.defines_strategy_with_reusable_values
+@deprecated_posargs
 def domains(
-    max_length=255,  # type: int
-    max_element_length=63,  # type: int
-):
-    # type: (...) -> SearchStrategy[Text]
+    *, max_length: int = 255, max_element_length: int = 63
+) -> SearchStrategy[str]:
     """Generate :rfc:`1035` compliant fully qualified domain names."""
     return DomainNameStrategy(
         max_length=max_length, max_element_length=max_element_length
@@ -153,8 +146,7 @@ def domains(
 
 
 @st.defines_strategy_with_reusable_values
-def urls():
-    # type: () -> SearchStrategy[Text]
+def urls() -> SearchStrategy[str]:
     """A strategy for :rfc:`3986`, generating http/https URLs."""
 
     def url_encode(s):
@@ -165,28 +157,25 @@ def urls():
     paths = st.lists(st.text(string.printable).map(url_encode)).map("/".join)
 
     return st.builds(
-        u"{}://{}{}/{}".format, schemes, domains(), st.just(u"") | ports, paths
+        "{}://{}{}/{}".format, schemes, domains(), st.just("") | ports, paths
     )
 
 
 @st.defines_strategy_with_reusable_values
-def ip4_addr_strings():
-    # type: () -> SearchStrategy[Text]
-    """A strategy for IPv4 address strings.
-
-    This consists of four strings representing integers [0..255],
-    without zero-padding, joined by dots.
-    """
-    return st.builds(u"{}.{}.{}.{}".format, *(4 * [st.integers(0, 255)]))
+def ip4_addr_strings() -> SearchStrategy[str]:
+    note_deprecation(
+        "Use `ip_addresses(v=4).map(str)` instead of `ip4_addr_strings()`; "
+        "the provisional strategy is less flexible and will be removed.",
+        since="2020-01-21",
+    )
+    return ip_addresses(v=4).map(str)
 
 
 @st.defines_strategy_with_reusable_values
-def ip6_addr_strings():
-    # type: () -> SearchStrategy[Text]
-    """A strategy for IPv6 address strings.
-
-    This consists of sixteen quads of hex digits (0000 .. FFFF), joined
-    by colons.  Values do not currently have zero-segments collapsed.
-    """
-    part = st.integers(0, 2 ** 16 - 1).map(u"{:04x}".format)
-    return st.tuples(*[part] * 8).map(lambda a: u":".join(a).upper())
+def ip6_addr_strings() -> SearchStrategy[str]:
+    note_deprecation(
+        "Use `ip_addresses(v=6).map(str)` instead of `ip6_addr_strings()`; "
+        "the provisional strategy is less flexible and will be removed.",
+        since="2020-01-21",
+    )
+    return ip_addresses(v=6).map(str)
